@@ -63,7 +63,8 @@ def stringify(line: str | bytes):
 
 
 class DockerOperator(BaseOperator):
-    """Execute a command inside a docker container.
+    """
+    Execute a command inside a docker container.
 
     By default, a temporary directory is
     created on the host and mounted into a container to allow storing files
@@ -433,7 +434,8 @@ class DockerOperator(BaseOperator):
             for log_chunk in logstream:
                 log_chunk = stringify(log_chunk).strip()
                 log_lines.append(log_chunk)
-                self.log.info("%s", log_chunk)
+                for log_chunk_line in log_chunk.split("\n"):
+                    self.log.info("%s", log_chunk_line)
 
             result = self.cli.wait(self.container["Id"])
             if result["StatusCode"] in self.skip_on_exit_code:
@@ -464,33 +466,33 @@ class DockerOperator(BaseOperator):
                 self.cli.remove_container(self.container["Id"], force=True)
 
     def _attempt_to_retrieve_result(self):
-        """Attempt to pull the result from the expected file.
+        """
+        Attempt to pull the result from the expected file.
 
         This uses Docker's ``get_archive`` function. If the file is not yet
         ready, *None* is returned.
         """
-
-        def copy_from_docker(container_id, src):
-            archived_result, stat = self.cli.get_archive(container_id, src)
-            if stat["size"] == 0:
-                # 0 byte file, it can't be anything else than None
-                return None
-            # no need to port to a file since we intend to deserialize
-            with BytesIO(b"".join(archived_result)) as f:
-                tar = tarfile.open(fileobj=f)
-                file = tar.extractfile(stat["name"])
-                lib = getattr(self, "pickling_library", pickle)
-                return lib.load(file)
-
         try:
-            return copy_from_docker(self.container["Id"], self.retrieve_output_path)
+            return self._copy_from_docker(self.container["Id"], self.retrieve_output_path)
         except APIError:
             return None
 
+    def _copy_from_docker(self, container_id, src):
+        archived_result, stat = self.cli.get_archive(container_id, src)
+        if stat["size"] == 0:
+            # 0 byte file, it can't be anything else than None
+            return None
+        # no need to port to a file since we intend to deserialize
+        with BytesIO(b"".join(archived_result)) as f:
+            tar = tarfile.open(fileobj=f)
+            file = tar.extractfile(stat["name"])
+            lib = getattr(self, "pickling_library", pickle)
+            return lib.load(file)
+
     def execute(self, context: Context) -> list[str] | str | None:
         # Pull the docker image if `force_pull` is set or image does not exist locally
         if self.force_pull or not self.cli.images(name=self.image):
-            self.log.info("Pulling docker image %s", self.image)
+            self.log.info("::group::Pulling docker image %s", self.image)
             latest_status: dict[str, str] = {}
             for output in self.cli.pull(self.image, stream=True, decode=True):
                 if isinstance(output, str):
@@ -506,11 +508,13 @@ class DockerOperator(BaseOperator):
                     if latest_status.get(output_id) != output_status:
                         self.log.info("%s: %s", output_id, output_status)
                         latest_status[output_id] = output_status
+            self.log.info("::endgroup::")
         return self._run_image()
 
     @staticmethod
     def format_command(command: list[str] | str | None) -> list[str] | str | None:
-        """Retrieve command(s).
+        """
+        Retrieve command(s).
 
         If command string starts with ``[``, the string is treated as a Python
         literal and parsed into a list of commands.
@@ -533,7 +537,8 @@ class DockerOperator(BaseOperator):
 
     @staticmethod
     def unpack_environment_variables(env_str: str) -> dict:
-        r"""Parse environment variables from the string.
+        r"""
+        Parse environment variables from the string.
 
         :param env_str: environment variables in the ``{key}={value}`` format,
             separated by a ``\n`` (newline)
